prefix = /usr/local
confflag = --prefix=$(prefix)
bdepends = squashfs-tools-4.x compiletc coreutils
delfiles = install.stamp build.stamp patch.stamp configure.stamp extract.stamp files.txt

export PKG_CONFIG_PATH = /usr/local/lib/pkgconfig:/usr/lib/pkgconfig
export CFLAGS = -march=i486 -mtune=i686 -Os -pipe
export CXXFLAGS = -march=i486 -mtune=i686 -Os -pipe

CAT      = busybox cat
WHOAMI   = busybox whoami
TOUCH    = busybox touch
TAR      = busybox tar
SORT     = busybox sort
SED      = busybox sed
RM       = busybox rm
BASENAME = busybox basename
MKDIR    = busybox mkdir
CP       = busybox cp
MD5SUM   = busybox md5sum
FIND     = /usr/local/bin/find

.PHONY: all
all: build.stamp

.PHONY: build-dep
build-dep:
	@for i in $(bdepends); do \
		if [ ! -f /usr/local/tce.installed/$$i ]; then \
			tcedir=`$(CAT) /opt/.tce_dir`; \
			if [ ! -f $$tcedir/optional/$$i.tcz ]; then \
				if [ `$(WHOAMI)` = "root" ]; then \
					sudo -u tc tce-load -w $$i; \
				else \
					tce-load -w $$i; \
				fi; \
			fi; \
			if [ `$(WHOAMI)` = "root" ]; then \
				sudo -u tc tce-load -i $$i; \
			else \
				tce-load -i $$i; \
			fi; \
			if [ ! -f /usr/local/tce.installed/$$i ]; then \
				exit 1; \
			fi; \
		fi; \
	done

extract.stamp: $(distfile)
	$(MAKE) extract
	$(TOUCH) extract.stamp

patch.stamp: extract.stamp
	@for i in $(patches); do \
		if [ -f $$i ]; then \
			cd $(workdir); \
			patch -p 1 < ../$$i; \
			[ "$$?" != "0" ] && exit 1; \
			cd ..; \
		fi; \
	done
	$(TOUCH) patch.stamp

configure.stamp: patch.stamp
	$(MAKE) build-dep
	$(MAKE) configure
	$(TOUCH) configure.stamp

build.stamp: configure.stamp
	$(MAKE) build-dep
	$(MAKE) build
	$(TOUCH) build.stamp

install.stamp: build.stamp
	$(TOUCH) install.stamp
	$(MAKE) install	
	$(FIND) /usr/local -newer install.stamp -not -type d | $(SORT) | $(SED) -e "s/^\///g" > files.txt

.PHONY: package
package: install.stamp
	@for i in `$(FIND) . -maxdepth 1 -name "*.tcz.list"`; do \
		name=`$(BASENAME) $$i .tcz.list`; \
		echo "[$$name]"; \
		sudo $(MKDIR) -p $$name; \
		sudo $(TAR) cf - -T $$i -C / | sudo $(TAR) xf - -C $$name; \
		LIST=`sudo $(FIND) $$name -type f | $(SORT)`; \
		for j in $$LIST; do \
			f=`sudo file $$j`; \
			RET=`echo $$f | grep "ELF 32-bit LSB shared object,.*not stripped"`; \
			if [ -n "$$RET" ]; then echo "[strip] $$j"; sudo strip $$j; fi; \
			RET=`echo $$f | grep "ELF 32-bit LSB executable,.*not stripped"`; \
			if [ -n "$$RET" ]; then echo "[strip] $$j"; sudo strip $$j; fi \
		done; \
		[ -d files.$$name ] && sudo $(CP) -a files.$$name/* $$name; \
		[ -d $$name/usr/local/tce.installed ] && sudo chown -R root.staff $$name/usr/local/tce.installed; \
		[ -d $$name/usr/local/tce.installed ] && sudo chmod 775 $$name/usr/local/tce.installed; \
		$(MKDIR) -p packages; \
		[ -f packages/$$name.tcz ] && sudo $(RM) -f packages/$$name.tcz; \
		mksquashfs $$name packages/$$name.tcz -b 4096; \
		cd $$name; $(FIND) . -not -type d | $(SORT) | $(SED) -e "s/^\.\///" > ../packages/$$name.tcz.list; cd ..; \
		cd packages; $(MD5SUM) $$name.tcz > $$name.tcz.md5.txt; cd ..; \
		[ -f $$name.tcz.dep ] && $(CP) -f $$name.tcz.dep packages/$$name.tcz.dep; \
		[ -f $$name.tcz.info ] && $(CP) -f $$name.tcz.info packages/$$name.tcz.info; \
		sudo $(RM) -rf $$name; \
		sudo $(TOUCH) /usr/local/tce.installed/$$name; \
	done

.PHONY: clean
clean:
	sudo $(RM) -rf packages $(workdir) $(delfiles)
